@@ -68,9 +68,12 @@ class TextColumn(NumericStatsMixin, BaseColumnPrimitiveTypeProfiler):
         profile = dict(
             min=self.min,
             max=self.max,
+            sum=self.sum,
             mean=self.mean,
             variance=self.variance,
             stddev=self.stddev,
+            skewness=self.skewness,
+            kurtosis=self.kurtosis,
             histogram=self._get_best_histogram_for_profile(),
             quantiles=self.quantiles,
             vocab=self.vocab,
