@@ -2,6 +2,7 @@ import os
 import unittest
 from unittest import mock
 
+import numpy as np
 import pandas as pd
 
 from dataprofiler import Data, ProfilerOptions, Profiler
@@ -27,25 +28,47 @@ class TestProfilerOptions(unittest.TestCase):
         for column in profile.options.properties:
             self.assertTrue(profile.options.properties[column].is_enabled)
 
-        for column in ["int", "float", "text"]:
-            column = profile.options.properties[column]
+        for column_type in ["int", "float", "text"]:
+            column = profile.options.properties[column_type]
             self.assertTrue(column.properties["histogram_and_quantiles"])
             self.assertTrue(column.properties["min"])
             self.assertTrue(column.properties["max"])
             self.assertTrue(column.properties["sum"])
             self.assertTrue(column.properties["variance"])
             self.assertTrue(column.properties["is_numeric_stats_enabled"])
-
+            if column_type != "text":
+                self.assertTrue(column.properties["num_zeros"].is_enabled)
+                self.assertTrue(column.properties["num_negatives"].is_enabled)
+            else:
+                self.assertFalse(column.properties["num_zeros"].is_enabled)
+                self.assertFalse(column.properties["num_negatives"].is_enabled)
         # Using ProfilerOptions with the default options
         options = ProfilerOptions()
         profile2 = Profiler(self.data, options=options)
         # Stored in Profiler as StructuredOptions
         self.assertEqual(profile2.options, options.structured_options)
 
+    def test_set_failures(self, *mocks):
+
+        options = ProfilerOptions()
+
+        # check if no '*.' it raises an error bc attribute not found
+        expected_error = ("type object 'ProfilerOptions' has no attribute "
+                          "'is_enabled'")
+        with self.assertRaisesRegex(AttributeError, expected_error):
+            options.set({"is_enabled": False})
+
+        # check if attribute doesn't exist, it raises an error
+        expected_error = ("type object 'structured_options' has no attribute "
+                          "'test'")
+        with self.assertRaisesRegex(AttributeError, expected_error):
+            options.set({"structured_options.test": False})
+
     def test_numerical_stats_option(self, *mocks):
         # Assert that the stats are disabled
         options = ProfilerOptions()
-        options.set({"is_numeric_stats_enabled": False})
+        options.set({"*.is_numeric_stats_enabled": False,
+                     "bias_correction.is_enabled": False})
         profile = Profiler(self.data, options=options)
 
         for column_name in profile.profile.keys():
@@ -59,11 +82,14 @@ class TestProfilerOptions(unittest.TestCase):
                     profile_column["statistics"]["histogram"]["bin_edges"])
                 self.assertIsNone(profile_column["statistics"]["min"])
                 self.assertIsNone(profile_column["statistics"]["max"])
-                self.assertEqual(0, profile_column["statistics"]["variance"])
+                self.assertTrue(np.isnan(profile_column["statistics"]["variance"]))
                 self.assertIsNone(profile_column["statistics"]["quantiles"][0])
+                self.assertTrue(np.isnan(profile_column["statistics"]["skewness"]))
+                self.assertTrue(np.isnan(profile_column["statistics"]["kurtosis"]))
 
         # Assert that the stats are enabled
-        options.set({"is_numeric_stats_enabled": True})
+        options.set({"*.is_numeric_stats_enabled": True,
+                     "bias_correction.is_enabled": True})
         profile = Profiler(self.data, options=options)
 
         for column_name in profile.profile.keys():
@@ -77,9 +103,11 @@ class TestProfilerOptions(unittest.TestCase):
                     profile_column["statistics"]["histogram"]["bin_edges"])
                 self.assertIsNotNone(profile_column["statistics"]["min"])
                 self.assertIsNotNone(profile_column["statistics"]["max"])
-                self.assertNotEqual(0, profile_column["statistics"]["variance"])
+                self.assertEqual(0.5, profile_column["statistics"]["variance"])
                 self.assertIsNotNone(
                     profile_column["statistics"]["quantiles"][0])
+                self.assertTrue(profile_column["statistics"]["skewness"] is np.nan)
+                self.assertTrue(profile_column["statistics"]["kurtosis"] is np.nan)
 
     def test_disable_labeler_in_profiler_options(self, *mocks):
         options = ProfilerOptions()
@@ -138,7 +166,11 @@ class TestProfilerOptions(unittest.TestCase):
             "min.is_enabled": False,
             "max.is_enabled": False,
             "sum.is_enabled": False,
-            "variance.is_enabled": False
+            "variance.is_enabled": False,
+            "skewness.is_enabled": False,
+            "kurtosis.is_enabled": False,
+            "num_zeros.is_enabled": False,
+            "num_negatives.is_enabled": False,
         }
         options.set(statistical_options)
 
@@ -147,7 +179,8 @@ class TestProfilerOptions(unittest.TestCase):
         float_options = options.structured_options.float.properties
         int_options = options.structured_options.int.properties
         for option in ["histogram_and_quantiles", "min", "max", "sum",
-                          "variance"]:
+                       "variance", "skewness", "kurtosis",
+                       "num_zeros", "num_negatives"]:
             self.assertFalse(text_options[option].is_enabled)
             self.assertFalse(float_options[option].is_enabled)
             self.assertFalse(int_options[option].is_enabled)
@@ -167,8 +200,10 @@ class TestProfilerOptions(unittest.TestCase):
                     profile_column["statistics"]["histogram"]["bin_edges"])
                 self.assertIsNone(profile_column["statistics"]["min"])
                 self.assertIsNone(profile_column["statistics"]["max"])
-                self.assertEqual(0, profile_column["statistics"]["variance"])
+                self.assertTrue(np.isnan(profile_column["statistics"]["variance"]))
                 self.assertIsNone(profile_column["statistics"]["quantiles"][0])
+                self.assertTrue(profile_column["statistics"]["skewness"] is np.nan)
+                self.assertTrue(profile_column["statistics"]["kurtosis"] is np.nan)
 
     def test_validate(self, *mocks):
         options = ProfilerOptions()
@@ -194,7 +229,9 @@ class TestProfilerOptions(unittest.TestCase):
             "min.is_enabled": False,
             "max.is_enabled": False,
             "sum.is_enabled": False,
-            "variance.is_enabled": True
+            "variance.is_enabled": True,
+            "skewness.is_enabled": False,
+            "kurtosis.is_enabled": False
         }
         # Asserts error since sum must be toggled on if variance is
         expected_error = (
@@ -212,7 +249,7 @@ class TestProfilerOptions(unittest.TestCase):
 
         # test warns if is_numeric_stats_enabled = False
         numerical_options = {
-            "is_numeric_stats_enabled": False,
+            "*.is_numeric_stats_enabled": False,
         }
         options.set(numerical_options)
         with self.assertWarnsRegex(UserWarning,
@@ -225,10 +262,11 @@ class TestProfilerOptions(unittest.TestCase):
         options = ProfilerOptions()
 
         # Ensure set works appropriately
-        options.set({"data_labeler.is_enabled": False,
-                     "min.is_enabled": False,
-                     "structured_options.data_labeler_dirpath": "test",
-                     "max_sample_size": 15})
+        options.set({
+            "data_labeler.is_enabled": False,
+            "min.is_enabled": False,
+            "structured_options.data_labeler.data_labeler_dirpath": "test",
+            "data_labeler.max_sample_size": 15})
 
         text_options = options.structured_options.text.properties
         float_options = options.structured_options.float.properties
@@ -260,7 +298,7 @@ class TestProfilerOptions(unittest.TestCase):
         float_options = FloatOptions()
         float_options.set({"precision.is_enabled": False,
                            "min.is_enabled": False,
-                           "is_enabled": False})
+                           "*.is_enabled": False})
 
         self.assertFalse(float_options.precision.is_enabled)
         self.assertFalse(float_options.min.is_enabled)
@@ -276,8 +314,7 @@ class TestProfilerOptions(unittest.TestCase):
                 ValueError, "ProfilerOptions.structured_options.text.max."
                             "is_enabled must be a Boolean."):
             profile_options = ProfilerOptions()
-            profile_options.structured_options.text.max.is_enabled \
-                = "String"
+            profile_options.structured_options.text.max.is_enabled = "String"
             profile_options.validate()
 
     def test_invalid_options_type(self, *mocks):
@@ -387,6 +424,21 @@ class TestProfilerOptions(unittest.TestCase):
         self.assertFalse(options.structured_options.text.vocab.is_enabled)
         self.assertTrue(options.unstructured_options.text.vocab.is_enabled)
 
+    def test_eq(self, *mocks):
+        options = ProfilerOptions()
+        options2 = ProfilerOptions()
+        options.unstructured_options.data_labeler.is_enabled = False
+        self.assertNotEqual(options, options2)
+        options2.unstructured_options.data_labeler.is_enabled = False
+        self.assertEqual(options, options2)
+
+        options.structured_options.float.precision.sample_ratio = 0.1
+        self.assertNotEqual(options, options2)
+        options2.structured_options.float.precision.sample_ratio = 0.15
+        self.assertNotEqual(options, options2)
+        options.structured_options.float.precision.sample_ratio = 0.1
+        self.assertNotEqual(options, options2)
+
 
 @mock.patch('dataprofiler.profilers.data_labeler_column_profile.'
             'DataLabelerColumn.update', return_value=None)
