@@ -5,6 +5,7 @@ from collections import defaultdict
 
 import pandas as pd
 import numpy as np
+import warnings
 
 from dataprofiler.profilers import IntColumn
 from dataprofiler.profilers.profiler_options import IntOptions
@@ -23,8 +24,11 @@ class TestIntColumn(unittest.TestCase):
         self.assertEqual(profiler.match_count, 0)
         self.assertEqual(profiler.min, None)
         self.assertEqual(profiler.max, None)
+        self.assertEqual(profiler.sum, 0)
         self.assertEqual(profiler.mean, 0)
-        self.assertEqual(profiler.variance, 0)
+        self.assertTrue(profiler.variance is np.nan)
+        self.assertTrue(profiler.skewness is np.nan)
+        self.assertTrue(profiler.kurtosis is np.nan)
         self.assertTrue(profiler.stddev is np.nan)
         self.assertIsNone(profiler.histogram_selection)
         self.assertDictEqual({k: profiler.quantiles.get(k, 'fail')
@@ -36,12 +40,14 @@ class TestIntColumn(unittest.TestCase):
         profiler = IntColumn(data.name)
         profiler.update(data)
         self.assertEqual(profiler.match_count, 1)
+        self.assertEqual(profiler.sum, 1)
         self.assertEqual(profiler.mean, 1)
-        self.assertEqual(profiler.variance, 0)
+        self.assertTrue(profiler.variance is np.nan)
 
         data = pd.Series([2])
         profiler.update(data)
         self.assertEqual(profiler.match_count, 2)
+        self.assertEqual(profiler.sum, 3)
         self.assertEqual(profiler.mean, 1.5)
         self.assertEqual(profiler.variance, 0.5)
 
@@ -198,8 +204,128 @@ class TestIntColumn(unittest.TestCase):
 
         df = pd.concat([df1, df2_ints, df3_ints])
         self.assertEqual(mean(df), num_profiler.mean)
-        self.assertEqual(variance, num_profiler.variance)
-        self.assertEqual(np.sqrt(variance), num_profiler.stddev)
+        self.assertAlmostEqual(variance, num_profiler.variance)
+        self.assertAlmostEqual(np.sqrt(variance), num_profiler.stddev)
+
+    def test_profiled_skewness(self):
+        data = np.linspace(-5, 5, 11).tolist()
+        df1 = pd.Series(data)
+
+        data = np.linspace(-3, 2, 11).tolist()
+        df2 = pd.Series(data)
+
+        data = np.full((10,), 1)
+        df3 = pd.Series(data)
+
+        num_profiler = IntColumn(df1.name)
+        num_profiler.update(df1.apply(str))
+
+        self.assertEqual(0, num_profiler.skewness)
+
+        df2_ints = df2[df2 == df2.round()]
+        num_profiler.update(df2.apply(str))
+        df = pd.concat([df1, df2_ints])
+        self.assertAlmostEqual(11 * np.sqrt(102 / 91) / 91, num_profiler.skewness)
+
+        df3_ints = df3[df3 == df3.round()]
+        num_profiler.update(df3.apply(str))
+        df = pd.concat([df1, df2_ints, df3_ints])
+        self.assertAlmostEqual(-6789 * np.sqrt(39 / 463) / 4630, num_profiler.skewness)
+
+    def test_profiled_kurtosis(self):
+        data = np.linspace(-5, 5, 11).tolist()
+        df1 = pd.Series(data)
+
+        data = np.linspace(-3, 2, 11).tolist()
+        df2 = pd.Series(data)
+
+        data = np.full((10,), 1)
+        df3 = pd.Series(data)
+
+        num_profiler = IntColumn(df1.name)
+        num_profiler.update(df1.apply(str))
+
+        self.assertAlmostEqual(-6 / 5, num_profiler.kurtosis)
+
+        df2_ints = df2[df2 == df2.round()]
+        num_profiler.update(df2.apply(str))
+        df = pd.concat([df1, df2_ints])
+        self.assertAlmostEqual(-29886 / 41405, num_profiler.kurtosis)
+
+        df3_ints = df3[df3 == df3.round()]
+        num_profiler.update(df3.apply(str))
+        df = pd.concat([df1, df2_ints, df3_ints])
+        self.assertAlmostEqual(16015779 / 42873800, num_profiler.kurtosis)
+
+    def test_bias_correction_option(self):
+        data = np.linspace(-5, 5, 11).tolist()
+        df1 = pd.Series(data)
+
+        data = np.linspace(-3, 2, 11).tolist()
+        df2 = pd.Series(data)
+
+        data = np.full((10,), 1)
+        df3 = pd.Series(data)
+
+        # Disable bias correction
+        options = IntOptions(); options.bias_correction.is_enabled = False
+        num_profiler = IntColumn(df1.name, options=options)
+        num_profiler.update(df1.apply(str))
+        self.assertAlmostEqual(10, num_profiler.variance)
+        self.assertAlmostEqual(0, num_profiler.skewness)
+        self.assertAlmostEqual(89/50 - 3, num_profiler.kurtosis)
+
+        df2_ints = df2[df2 == df2.round()]
+        num_profiler.update(df2.apply(str))
+        df = pd.concat([df1, df2_ints])
+        self.assertAlmostEqual(2184 / 289, num_profiler.variance)
+        self.assertAlmostEqual(165 * np.sqrt(3 / 182) / 182, num_profiler.skewness)
+        self.assertAlmostEqual(60769 / 28392 - 3, num_profiler.kurtosis)
+
+        df3_ints = df3[df3 == df3.round()]
+        num_profiler.update(df3.apply(str))
+        df = pd.concat([df1, df2_ints, df3_ints])
+        self.assertAlmostEqual(3704 / 729, num_profiler.variance)
+        self.assertAlmostEqual(-11315 / (926 * np.sqrt(926)), num_profiler.skewness)
+        self.assertAlmostEqual(5305359 / 1714952 - 3, num_profiler.kurtosis)
+
+    def test_bias_correction_merge(self):
+        data = np.linspace(-5, 5, 11).tolist()
+        df1 = pd.Series(data)
+
+        data = np.linspace(-3, 2, 11).tolist()
+        df2 = pd.Series(data)
+
+        data = np.full((10,), 1)
+        df3 = pd.Series(data)
+
+        # Disable bias correction
+        options = IntOptions(); options.bias_correction.is_enabled = False
+        num_profiler1 = IntColumn(df1.name, options=options)
+        num_profiler1.update(df1.apply(str))
+        self.assertAlmostEqual(10, num_profiler1.variance)
+        self.assertAlmostEqual(0, num_profiler1.skewness)
+        self.assertAlmostEqual(89/50 - 3, num_profiler1.kurtosis)
+
+        df2_ints = df2[df2 == df2.round()]
+        num_profiler2 = IntColumn(df2.name)
+        num_profiler2.update(df2.apply(str))
+        num_profiler_merged = num_profiler1 + num_profiler2
+        # Values should stay biased values
+        self.assertFalse(num_profiler_merged.bias_correction)
+        self.assertAlmostEqual(2184 / 289, num_profiler_merged.variance)
+        self.assertAlmostEqual(165 * np.sqrt(3 / 182) / 182,
+                               num_profiler_merged.skewness)
+        self.assertAlmostEqual(60769 / 28392 - 3, num_profiler_merged.kurtosis)
+
+        df3_ints = df3[df3 == df3.round()]
+        num_profiler3 = IntColumn(df3.name)
+        num_profiler3.update(df3.apply(str))
+        num_profiler_merged = num_profiler1 + num_profiler2 + num_profiler3
+        self.assertFalse(num_profiler_merged.bias_correction)
+        self.assertAlmostEqual(3704 / 729, num_profiler_merged.variance)
+        self.assertAlmostEqual(-11315 / (926 * np.sqrt(926)), num_profiler_merged.skewness)
+        self.assertAlmostEqual(5305359 / 1714952 - 3, num_profiler_merged.kurtosis)
 
     def test_profiled_histogram(self):
         """
@@ -269,8 +395,13 @@ class TestIntColumn(unittest.TestCase):
         expected_profile = dict(
             min=2.0,
             max=6.0,
+            sum=8.0,
             mean=4.0,
             variance=8.0,
+            num_zeros = 0,
+            num_negatives = 0,
+            skewness=np.nan,
+            kurtosis=np.nan,
             stddev=np.sqrt(8.0),
             histogram={
                 'bin_counts': np.array([1, 0, 1]),
@@ -283,7 +414,9 @@ class TestIntColumn(unittest.TestCase):
             },
             times=defaultdict(
                 float, {'histogram_and_quantiles': 1.0, 'max': 1.0, 'min': 1.0,
-                        'sum': 1.0, 'variance': 1.0})
+                        'sum': 1.0, 'variance': 1.0, 'skewness': 1.0,
+                        'kurtosis': 1.0, 'num_negatives': 1.0,
+                        'num_zeros': 1.0})
             
         )
         time_array = [float(i) for i in range(100, 0, -1)]
@@ -313,7 +446,10 @@ class TestIntColumn(unittest.TestCase):
 
             expected = defaultdict(
                 float, {'min': 1.0, 'max': 1.0, 'sum': 1.0, 'variance': 1.0,
-                        'histogram_and_quantiles': 1.0})
+                        'skewness': 1.0, 'kurtosis': 1.0,
+                        'histogram_and_quantiles': 1.0,
+                        'num_negatives': 1.0, 'num_zeros': 1.0,
+                        })
             self.assertEqual(expected, profile['times'])
 
             # Validate time in datetime class has expected time after second
@@ -321,7 +457,9 @@ class TestIntColumn(unittest.TestCase):
             profiler.update(df)
             expected = defaultdict(
                 float, {'min': 2.0, 'max': 2.0, 'sum': 2.0, 'variance': 2.0,
-                        'histogram_and_quantiles': 2.0})
+                        'skewness': 2.0, 'kurtosis': 2.0,
+                        'histogram_and_quantiles': 2.0, 'num_negatives': 2.0,
+                        'num_zeros': 2.0,})
             self.assertEqual(expected, profiler.profile['times'])
 
     def test_option_timing(self):
@@ -342,13 +480,19 @@ class TestIntColumn(unittest.TestCase):
             # Validate the time in the datetime class has the expected time.
             profile = profiler.profile
 
-            expected = defaultdict(float, {'max': 1.0, 'sum': 1.0, 'variance': 1.0, \
+            expected = defaultdict(float, {'max': 1.0, 'sum': 1.0, 'variance': 1.0,
+                                           'skewness': 1.0, 'kurtosis': 1.0,
+                                           'num_zeros': 1.0,
+                                           'num_negatives': 1.0,
                                            'histogram_and_quantiles': 1.0})
             self.assertCountEqual(expected, profile['times'])
 
             # Validate time in datetime class has expected time after second update
             profiler.update(df)
-            expected = defaultdict(float, {'max': 2.0, 'sum': 2.0, 'variance': 2.0, \
+            expected = defaultdict(float, {'max': 2.0, 'sum': 2.0, 'variance': 2.0,
+                                           'skewness': 2.0, 'kurtosis': 2.0,
+                                           'num_zeros': 2.0,
+                                           'num_negatives': 2.0,
                                            'histogram_and_quantiles': 2.0})
             self.assertCountEqual(expected, profiler.profile['times'])
 
@@ -366,8 +510,11 @@ class TestIntColumn(unittest.TestCase):
         expected_profile = dict(
             min=2.0,
             max=15.0,
+            sum=33,
             mean=8.25,
             variance=30.916666666666668,
+            skewness=918 * np.sqrt(3 / 371) / 371,
+            kurtosis=-16068/19663,
             stddev=np.sqrt(30.916),
             histogram={
                 'bin_counts': np.array([1, 1, 1, 1]),
@@ -381,19 +528,47 @@ class TestIntColumn(unittest.TestCase):
         profile3 = profiler3.profile
         histogram = profile3.pop('histogram')
 
+        self.assertTrue(profiler3.bias_correction)
         self.assertAlmostEqual(profiler3.stddev,
                                expected_profile.pop('stddev'),places=3)
         self.assertAlmostEqual(profiler3.variance,
                                expected_profile.pop('variance'), places=3)
+        self.assertAlmostEqual(profiler3.skewness,
+                               expected_profile.pop('skewness'),places=3)
+        self.assertAlmostEqual(profiler3.kurtosis,
+                               expected_profile.pop('kurtosis'), places=3)
         self.assertEqual(profiler3.mean,expected_profile.pop('mean'))
         self.assertEqual(profiler3.histogram_selection, 'doane')
-        self.assertEqual(profiler3.min,expected_profile.pop('min'))
-        self.assertEqual(profiler3.max,expected_profile.pop('max'))
+        self.assertEqual(profiler3.min, expected_profile.pop('min'))
+        self.assertEqual(profiler3.max, expected_profile.pop('max'))
+        self.assertEqual(profiler3.sum, expected_profile.pop('sum'))
         self.assertEqual(histogram['bin_counts'].tolist(),
                          expected_histogram['bin_counts'].tolist())
         self.assertCountEqual(histogram['bin_edges'],
                               expected_histogram['bin_edges'])
 
+    def test_profile_merge_for_zeros_and_negatives(self):
+        data = [2.0, 8.5, 'not an int', 6.0, -3, 0]
+        df = pd.Series(data).apply(str)
+        profiler1 = IntColumn("Int")
+        profiler1.update(df)
+
+        data2 = [0.0, 3.5, 'not an int', 125.0, 0, -0.1, -88]
+        df2 = pd.Series(data2).apply(str)
+        profiler2 = IntColumn("Int")
+        profiler2.update(df2)
+
+        expected_profile = dict(
+            num_zeros=3,
+            num_negatives=2
+        )
+
+        profiler3 = profiler1 + profiler2
+
+        self.assertEqual(profiler3.num_zeros, expected_profile.pop('num_zeros'))
+        self.assertEqual(profiler3.num_negatives,
+                         expected_profile.pop('num_negatives'))
+
     def test_profile_merge_edge_case(self):
         data = [2.0, 12.5, 'not a float', 6.0, 'not a float']
         df = pd.Series(data).apply(str)
@@ -421,6 +596,8 @@ class TestIntColumn(unittest.TestCase):
         profiler = profiler1 + profiler2
         self.assertEqual(profiler.min, None)
         self.assertEqual(profiler.max, None)
+        self.assertTrue(np.isnan(profiler.skewness))
+        self.assertTrue(np.isnan(profiler.kurtosis))
         self.assertIsNone(profiler.histogram_selection)
 
         df3 = pd.Series([2, 3]).apply(str)
@@ -430,6 +607,10 @@ class TestIntColumn(unittest.TestCase):
         profiler = profiler1 + profiler3
         self.assertEqual(profiler.min, 2)
         self.assertEqual(profiler.max, 3)
+        self.assertTrue(np.isnan(profiler.skewness))
+        self.assertTrue(np.isnan(profiler.kurtosis))
+        self.assertEqual(profiler.num_zeros, 0)
+        self.assertEqual(profiler.num_negatives, 0)
 
         df4 = pd.Series([4, 5]).apply(str)
         profiler4 = IntColumn("Int")
@@ -438,6 +619,21 @@ class TestIntColumn(unittest.TestCase):
         profiler = profiler3 + profiler4
         self.assertEqual(profiler.min, 2)
         self.assertEqual(profiler.max, 5)
+        self.assertEqual(profiler.skewness, 0)
+        self.assertAlmostEqual(profiler.kurtosis, -1.2)
+        self.assertEqual(profiler.num_zeros, 0)
+        self.assertEqual(profiler.num_negatives,0)
+
+        df5 = pd.Series([0, 0, -1]).apply(str)
+        profiler5 = IntColumn("Int")
+        profiler5.update(df5)
+
+        profiler = profiler4 + profiler5
+        self.assertEqual(profiler.min, -1)
+        self.assertEqual(profiler.max, 5)
+        self.assertEqual(profiler.num_zeros, 2)
+        self.assertEqual(profiler.num_negatives, 1)
+
 
     def test_custom_bin_count_merge(self):
 
@@ -594,3 +790,74 @@ class TestIntColumn(unittest.TestCase):
 
         profile_1 + profile_2
 
+    def test_insufficient_counts(self):
+        data = pd.Series(['1'])
+        profiler = IntColumn(data.name)
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+
+            profiler.update(data)
+            var = profiler.variance
+            skew = profiler.skewness
+            kurt = profiler.kurtosis
+            # Verify values are NaN
+            self.assertTrue(np.isnan(var))
+            self.assertTrue(np.isnan(skew))
+            self.assertTrue(np.isnan(kurt))
+            # Verify warning was raised properly
+            self.assertEqual(3, len(w))
+            for i in range(0, len(w)):
+                self.assertEqual(w[i].category, RuntimeWarning)
+                self.assertTrue("Insufficient match count to correct bias in" \
+                                in str(w[i].message))
+
+        # Update the data so that the match count is good
+        data2 = pd.Series(['-2', '-1', '1', '2'])
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+
+            profiler.update(data2)
+            var = profiler.variance
+            skew = profiler.skewness
+            kurt = profiler.kurtosis
+            # Verify values are no longer NaN
+            self.assertFalse(np.isnan(var))
+            self.assertFalse(np.isnan(skew))
+            self.assertFalse(np.isnan(kurt))
+            # Verify warning-related things. In this case, we check
+            # to make sure NO warnings were thrown since we have
+            # a sufficient match count.
+            self.assertEqual(0, len(w))
+
+    def test_diff(self):
+        """
+        Makes sure the IntColumn Diff() works appropriately.
+        """
+        data = [2, 'not an int', 6, 4]
+        df = pd.Series(data).apply(str)
+        profiler1 = IntColumn("Int")
+        profiler1.update(df)
+
+        data = [1, 15]
+        df = pd.Series(data).apply(str)
+        profiler2 = IntColumn("Int")
+        profiler2.update(df)
+
+        # Assert the difference report is correct
+        expected_diff = {
+            'max': -9.0,
+            'mean': -4.0,
+            'min': 1.0,
+            'stddev': -7.899494936611665,
+            'sum': -4.0,
+            'variance': -94.0
+        }
+        self.assertDictEqual(expected_diff, profiler1.diff(profiler2))
+        
+        # Assert type error is properly called
+        with self.assertRaises(TypeError) as exc:
+            profiler1.diff("Inproper input")
+        self.assertEqual(str(exc.exception),
+                         "Unsupported operand type(s) for diff: 'IntColumn' and"
+                         " 'str'")
